@@ -22,7 +22,6 @@ import tempfile
 import fixtures
 from keystoneauth1 import exceptions as ks_exceptions
 import mock
-from mistralclient.api import base as mistralclient_base
 from novaclient import exceptions
 from oslo_config import fixture as config_fixture
 from oslotest import base
@@ -54,9 +53,10 @@ class TestUndercloud(BaseTestCase):
     @mock.patch('instack_undercloud.undercloud._run_instack')
     @mock.patch('instack_undercloud.undercloud._generate_environment')
     @mock.patch('instack_undercloud.undercloud._load_config')
-    def test_install(self, mock_load_config, mock_generate_environment,
-                     mock_run_instack, mock_run_clean_all,
-                     mock_run_yum_update, mock_run_orc,
+    @mock.patch('instack_undercloud.undercloud._die_tuskar_die')
+    def test_install(self, mock_die_tuskar_die, mock_load_config,
+                     mock_generate_environment, mock_run_instack,
+                     mock_run_clean_all, mock_run_yum_update, mock_run_orc,
                      mock_post_config, mock_run_command,
                      mock_validate_configuration, mock_configure_logging,
                      mock_upgrade_fact):
@@ -70,6 +70,7 @@ class TestUndercloud(BaseTestCase):
         mock_run_command.assert_called_with(
             ['sudo', 'rm', '-f', '/tmp/svc-map-services'], None, 'rm')
         mock_upgrade_fact.assert_called_with(False)
+        mock_die_tuskar_die.assert_not_called()
 
     @mock.patch('instack_undercloud.undercloud._handle_upgrade_fact')
     @mock.patch('instack_undercloud.undercloud._configure_logging')
@@ -82,10 +83,11 @@ class TestUndercloud(BaseTestCase):
     @mock.patch('instack_undercloud.undercloud._run_instack')
     @mock.patch('instack_undercloud.undercloud._generate_environment')
     @mock.patch('instack_undercloud.undercloud._load_config')
-    def test_install_upgrade(self, mock_load_config, mock_generate_environment,
-                             mock_run_instack, mock_run_yum_clean_all,
-                             mock_run_yum_update, mock_run_orc,
-                             mock_post_config, mock_run_command,
+    @mock.patch('instack_undercloud.undercloud._die_tuskar_die')
+    def test_install_upgrade(self, mock_die_tuskar_die, mock_load_config,
+                             mock_generate_environment, mock_run_instack,
+                             mock_run_yum_clean_all, mock_run_yum_update,
+                             mock_run_orc, mock_post_config, mock_run_command,
                              mock_validate_configuration,
                              mock_configure_logging, mock_upgrade_fact):
         fake_env = mock.MagicMock()
@@ -98,6 +100,7 @@ class TestUndercloud(BaseTestCase):
         mock_run_command.assert_called_with(
             ['sudo', 'rm', '-f', '/tmp/svc-map-services'], None, 'rm')
         mock_upgrade_fact.assert_called_with(True)
+        mock_die_tuskar_die.assert_called_once()
 
     @mock.patch('instack_undercloud.undercloud._configure_logging')
     def test_install_exception(self, mock_configure_logging):
@@ -135,10 +138,10 @@ class TestUndercloud(BaseTestCase):
     def test_extract_from_stackrc(self):
         with open(os.path.expanduser('~/stackrc'), 'w') as f:
             f.write('OS_USERNAME=aturing\n')
-            f.write('OS_AUTH_URL=http://bletchley:5000/v2.0\n')
+            f.write('OS_AUTH_URL=http://bletchley:5000/\n')
         self.assertEqual('aturing',
                          undercloud._extract_from_stackrc('OS_USERNAME'))
-        self.assertEqual('http://bletchley:5000/v2.0',
+        self.assertEqual('http://bletchley:5000/',
                          undercloud._extract_from_stackrc('OS_AUTH_URL'))
 
     @mock.patch('instack_undercloud.undercloud._check_hostname')
@@ -419,6 +422,32 @@ class TestGenerateEnvironment(BaseTestCase):
                                            'pxe_ipmitool'])
         self.assertEqual(env['INSPECTION_NODE_NOT_FOUND_HOOK'], 'enroll')
 
+    def test_enabled_hardware_types(self):
+        conf = config_fixture.Config()
+        self.useFixture(conf)
+        conf.config(enable_node_discovery=True,
+                    discovery_default_driver='foobar',
+                    enabled_hardware_types=['ipmi', 'foobar'])
+        env = undercloud._generate_environment('.')
+        # The list is generated from a set, so we can't rely on ordering.
+        # Instead make sure that it looks like a valid list by parsing it.
+        drivers = json.loads(env['ENABLED_DRIVERS'])
+        hw_types = json.loads(env['ENABLED_HARDWARE_TYPES'])
+        # The driver is already in hardware types, so we don't try adding it to
+        # the driver list.
+        self.assertEqual(sorted(drivers), ['pxe_drac', 'pxe_ilo',
+                                           'pxe_ipmitool'])
+        self.assertEqual(sorted(hw_types), ['foobar', 'ipmi'])
+
+    def test_docker_registry_mirror(self):
+        conf = config_fixture.Config()
+        self.useFixture(conf)
+        conf.config(docker_registry_mirror='http://foo/bar')
+        env = undercloud._generate_environment('.')
+        # Spot check one service
+        self.assertEqual('http://foo/bar',
+                         env['DOCKER_REGISTRY_MIRROR'])
+
     def test_generate_endpoints(self):
         env = undercloud._generate_environment('.')
         endpoint_vars = {k: v for (k, v) in env.items()
@@ -447,7 +476,7 @@ class TestGenerateEnvironment(BaseTestCase):
                          env['UNDERCLOUD_ENDPOINT_KEYSTONE_INTERNAL'])
         self.assertEqual('http://192.168.24.3:35357',
                          env['UNDERCLOUD_ENDPOINT_KEYSTONE_ADMIN'])
-        self.assertEqual('https://192.168.24.2:443/keystone/v2.0',
+        self.assertEqual('https://192.168.24.2:443/keystone/v3',
                          env['UNDERCLOUD_ENDPOINT_KEYSTONE_UI_CONFIG_PUBLIC'])
         # Also check that the tenant id part is preserved
         self.assertEqual('https://192.168.24.2:13808/v1/AUTH_%(tenant_id)s',
@@ -635,6 +664,7 @@ class TestConfigureSshKeys(base.BaseTestCase):
 class TestPostConfig(base.BaseTestCase):
     @mock.patch('instack_undercloud.undercloud._member_role_exists')
     @mock.patch('novaclient.client.Client', autospec=True)
+    @mock.patch('swiftclient.client.Connection', autospec=True)
     @mock.patch('mistralclient.api.client.client', autospec=True)
     @mock.patch('instack_undercloud.undercloud._delete_default_flavors')
     @mock.patch('instack_undercloud.undercloud._copy_stackrc')
@@ -645,40 +675,45 @@ class TestPostConfig(base.BaseTestCase):
     def test_post_config(self, mock_post_config_mistral, mock_ensure_flavor,
                          mock_configure_ssh_keys, mock_get_auth_values,
                          mock_copy_stackrc, mock_delete, mock_mistral_client,
-                         mock_nova_client, mock_member_role_exists):
+                         mock_swift_client, mock_nova_client,
+                         mock_member_role_exists):
         instack_env = {
             'UNDERCLOUD_ENDPOINT_MISTRAL_PUBLIC':
                 'http://192.168.24.1:8989/v2',
         }
         mock_get_auth_values.return_value = ('aturing', '3nigma', 'hut8',
-                                             'http://bletchley:5000/v2.0')
-        mock_instance = mock.Mock()
-        mock_nova_client.return_value = mock_instance
+                                             'http://bletchley:5000/')
+        mock_instance_nova = mock.Mock()
+        mock_nova_client.return_value = mock_instance_nova
+        mock_instance_swift = mock.Mock()
+        mock_swift_client.return_value = mock_instance_swift
         mock_instance_mistral = mock.Mock()
         mock_mistral_client.return_value = mock_instance_mistral
         undercloud._post_config(instack_env)
         mock_nova_client.assert_called_with(
             2, 'aturing', '3nigma', project_name='hut8',
-            auth_url='http://bletchley:5000/v2.0')
+            auth_url='http://bletchley:5000/')
         self.assertTrue(mock_copy_stackrc.called)
-        mock_configure_ssh_keys.assert_called_with(mock_instance)
-        calls = [mock.call(mock_instance, 'baremetal'),
-                 mock.call(mock_instance, 'control', 'control'),
-                 mock.call(mock_instance, 'compute', 'compute'),
-                 mock.call(mock_instance, 'ceph-storage', 'ceph-storage'),
-                 mock.call(mock_instance, 'block-storage', 'block-storage'),
-                 mock.call(mock_instance, 'swift-storage', 'swift-storage'),
+        mock_configure_ssh_keys.assert_called_with(mock_instance_nova)
+        calls = [mock.call(mock_instance_nova, 'baremetal'),
+                 mock.call(mock_instance_nova, 'control', 'control'),
+                 mock.call(mock_instance_nova, 'compute', 'compute'),
+                 mock.call(mock_instance_nova, 'ceph-storage', 'ceph-storage'),
+                 mock.call(mock_instance_nova,
+                           'block-storage', 'block-storage'),
+                 mock.call(mock_instance_nova,
+                           'swift-storage', 'swift-storage'),
                  ]
         mock_ensure_flavor.assert_has_calls(calls)
-        mock_post_config_mistral.assert_called_once_with(instack_env,
-                                                         mock_instance_mistral)
+        mock_post_config_mistral.assert_called_once_with(
+            instack_env, mock_instance_mistral, mock_instance_swift)
 
     def test_create_default_plan(self):
         mock_mistral = mock.Mock()
         mock_mistral.environments.list.return_value = []
         mock_mistral.executions.get.return_value = mock.Mock(state="SUCCESS")
 
-        undercloud._create_default_plan(mock_mistral)
+        undercloud._create_default_plan(mock_mistral, [])
         mock_mistral.executions.create.assert_called_once_with(
             'tripleo.plan_management.v1.create_default_deployment_plan',
             workflow_input={
@@ -689,18 +724,13 @@ class TestPostConfig(base.BaseTestCase):
 
     def test_create_default_plan_existing(self):
         mock_mistral = mock.Mock()
-        environment = collections.namedtuple('environment', ['name'])
-        mock_mistral.environments.list.return_value = [
-            environment(name='overcloud')
-        ]
-
-        undercloud._create_default_plan(mock_mistral)
+        undercloud._create_default_plan(mock_mistral, ['overcloud'])
         mock_mistral.executions.create.assert_not_called()
 
     def test_create_config_environment(self):
         mock_mistral = mock.Mock()
         mock_mistral.environments.get.side_effect = (
-            mistralclient_base.APIException)
+            ks_exceptions.NotFound)
 
         env = {
             "UNDERCLOUD_CEILOMETER_SNMPD_PASSWORD": "snmpd-pass"
@@ -734,21 +764,19 @@ class TestPostConfig(base.BaseTestCase):
     @mock.patch('time.sleep')
     def test_create_default_plan_timeout(self, mock_sleep):
         mock_mistral = mock.Mock()
-        mock_mistral.environments.list.return_value = []
         mock_mistral.executions.get.return_value = mock.Mock(state="RUNNING")
 
         self.assertRaises(
             RuntimeError,
-            undercloud._create_default_plan, mock_mistral, timeout=0)
+            undercloud._create_default_plan, mock_mistral, [], timeout=0)
 
     def test_create_default_plan_failed(self):
         mock_mistral = mock.Mock()
-        mock_mistral.environments.list.return_value = []
         mock_mistral.executions.get.return_value = mock.Mock(state="ERROR")
 
         self.assertRaises(
             RuntimeError,
-            undercloud._create_default_plan, mock_mistral)
+            undercloud._create_default_plan, mock_mistral, [])
 
     @mock.patch('instack_undercloud.undercloud._run_command')
     def test_copy_stackrc(self, mock_run):
@@ -762,7 +790,7 @@ class TestPostConfig(base.BaseTestCase):
 
     def _mock_ksclient_roles(self, mock_auth_values, mock_ksdiscover, roles):
         mock_auth_values.return_value = ('user', 'password',
-                                         'tenant', 'http://test:123')
+                                         'project', 'http://test:123')
         mock_discover = mock.Mock()
         mock_ksdiscover.return_value = mock_discover
         mock_client = mock.Mock()
@@ -776,12 +804,14 @@ class TestPostConfig(base.BaseTestCase):
         mock_client.roles = mock_roles
         mock_discover.create_client.return_value = mock_client
 
-        mock_tenant_list = [mock.Mock(), mock.Mock()]
-        mock_tenant_list[0].name = 'admin'
-        mock_tenant_list[0].id = 'admin-id'
-        mock_tenant_list[1].name = 'service'
-        mock_tenant_list[1].id = 'service-id'
-        mock_client.tenants.list.return_value = mock_tenant_list
+        mock_client.version = 'v3'
+
+        mock_project_list = [mock.Mock(), mock.Mock()]
+        mock_project_list[0].name = 'admin'
+        mock_project_list[0].id = 'admin-id'
+        mock_project_list[1].name = 'service'
+        mock_project_list[1].id = 'service-id'
+        mock_client.projects.list.return_value = mock_project_list
 
         mock_user_list = [mock.Mock(), mock.Mock()]
         mock_user_list[0].name = 'admin'
@@ -799,7 +829,7 @@ class TestPostConfig(base.BaseTestCase):
                                                 mock_ksdiscover,
                                                 ['admin'])
         undercloud._member_role_exists()
-        self.assertFalse(mock_client.tenants.list.called)
+        self.assertFalse(mock_client.projects.list.called)
 
     @mock.patch('keystoneclient.discover.Discover')
     @mock.patch('instack_undercloud.undercloud._get_auth_values')
@@ -813,8 +843,8 @@ class TestPostConfig(base.BaseTestCase):
         undercloud._member_role_exists()
         mock_user = mock_client.users.list.return_value[0]
         mock_role = mock_client.roles.list.return_value[1]
-        mock_client.roles.add_user_role.assert_called_once_with(
-            mock_user, mock_role, 'admin-id')
+        mock_client.roles.grant.assert_called_once_with(
+            mock_role, user=mock_user, project='admin-id')
 
     @mock.patch('keystoneclient.discover.Discover')
     @mock.patch('instack_undercloud.undercloud._get_auth_values')
@@ -826,12 +856,12 @@ class TestPostConfig(base.BaseTestCase):
                                                 mock_ksdiscover,
                                                 ['admin', '_member_'])
         fake_exception = ks_exceptions.http.Conflict('test')
-        mock_client.roles.add_user_role.side_effect = fake_exception
+        mock_client.roles.grant.side_effect = fake_exception
         undercloud._member_role_exists()
         mock_user = mock_client.users.list.return_value[0]
         mock_role = mock_client.roles.list.return_value[1]
-        mock_client.roles.add_user_role.assert_called_once_with(
-            mock_user, mock_role, 'admin-id')
+        mock_client.roles.grant.assert_called_once_with(
+            mock_role, user=mock_user, project='admin-id')
 
     def _create_flavor_mocks(self):
         mock_nova = mock.Mock()
@@ -958,7 +988,7 @@ class TestInstackEnvironment(BaseTestCase):
         def _set():
             env['CATS_AND_DOGS_PATH'] = '/home'
 
-        self.assertRaisesRegexp(KeyError, 'CATS_AND_DOGS_PATH', _set)
+        self.assertRaisesRegex(KeyError, 'CATS_AND_DOGS_PATH', _set)
 
     def test_get_always_allowed(self):
         env = undercloud.InstackEnvironment()
