@@ -29,13 +29,13 @@ import sys
 import tempfile
 import time
 import uuid
+import yaml
 
-from keystoneclient import exceptions as ks_exceptions
-from keystoneclient import auth
-from keystoneclient import session
+from keystoneauth1 import session
+from keystoneauth1 import exceptions as ks_exceptions
 from keystoneclient import discover
+import keystoneauth1.identity.generic as ks_auth
 from mistralclient.api import client as mistralclient
-from mistralclient.api import base as mistralclient_base
 import novaclient as nc
 from novaclient import client as novaclient
 from novaclient import exceptions
@@ -43,6 +43,7 @@ from oslo_config import cfg
 import psutil
 import pystache
 import six
+from swiftclient import client as swiftclient
 
 from instack_undercloud import validator
 
@@ -220,7 +221,7 @@ _opts = [
                default='',
                help=('Path to hieradata override file. If set, the file will '
                      'be copied under /etc/puppet/hieradata and set as the '
-                     'first file in the hiera hierarchy. This can be used to '
+                     'first file in the hiera hierarchy. This can be used '
                      'to custom configure services beyond what '
                      'undercloud.conf provides')
                ),
@@ -296,9 +297,9 @@ _opts = [
                 help=('Whether to install Tempest in the Undercloud.')
                 ),
     cfg.BoolOpt('enable_telemetry',
-                default=True,
+                default=False,
                 help=('Whether to install Telemetry services '
-                      '(ceilometer, aodh) in the Undercloud.')
+                      '(ceilometer, gnocchi, aodh, panko ) in the Undercloud.')
                 ),
     cfg.BoolOpt('enable_ui',
                 default=True,
@@ -314,18 +315,30 @@ _opts = [
                 help=('Whether to install the Volume service. It is not '
                       'currently used in the undercloud.')),
     cfg.BoolOpt('enable_legacy_ceilometer_api',
-                default=True,
+                default=False,
                 help=('Whether to enable legacy ceilometer api '
                       'in the Undercloud. '
                       'Note: Ceilometer API has been deprecated and will be '
                       'removed in future release. Please consider moving to '
                       'gnocchi/Aodh/Panko API instead.')
                 ),
+    cfg.BoolOpt('enable_legacy_ceilometer_collector',
+                default=False,
+                help=('Whether to enable legacy ceilometer collector '
+                      'in the Undercloud. '
+                      'Note: Ceilometer collector has been deprecated and '
+                      'will be removed in future release.')
+                ),
     cfg.BoolOpt('enable_novajoin',
                 default=False,
                 help=('Whether to install novajoin metadata service in '
                       'the Undercloud.')
                 ),
+    cfg.BoolOpt('enable_container_images_build',
+                default=True,
+                help=('Whether to enable docker container images to be build '
+                      'on the undercloud.')
+                ),
     cfg.StrOpt('ipa_otp',
                default='',
                help=('One Time Password to register Undercloud node with '
@@ -349,14 +362,23 @@ _opts = [
                 help=('Whether to clean overcloud nodes (wipe the hard drive) '
                       'between deployments and after the introspection.')),
     cfg.ListOpt('enabled_drivers',
-                default=['pxe_ipmitool', 'pxe_drac', 'pxe_ilo', 'pxe_ssh'],
+                default=['pxe_ipmitool', 'pxe_drac', 'pxe_ilo'],
                 help=('List of enabled bare metal drivers.')),
+    cfg.ListOpt('enabled_hardware_types',
+                default=['ipmi', 'redfish'],
+                help=('List of enabled bare metal hardware types (next '
+                      'generation drivers).')),
+    cfg.StrOpt('docker_registry_mirror',
+               default='',
+               help=('An optional docker \'registry-mirror\' that will be'
+                     'configured in /etc/docker/daemon.json.')
+               ),
 ]
 
 # Passwords, tokens, hashes
 _auth_opts = [
     cfg.StrOpt('undercloud_db_password',
-               help=('Password used for MySQL root user. '
+               help=('Password used for MySQL databases. '
                      'If left unset, one will be automatically generated.')
                ),
     cfg.StrOpt('undercloud_admin_token',
@@ -471,6 +493,10 @@ _auth_opts = [
                help=('Cinder service password. '
                      'If left unset, one will be automatically generated.')
                ),
+    cfg.StrOpt('undercloud_novajoin_password',
+               help=('Novajoin vendordata plugin service password. '
+                     'If left unset, one will be automatically generated.')
+               ),
 ]
 CONF.register_opts(_opts)
 CONF.register_opts(_auth_opts, group='auth')
@@ -501,10 +527,11 @@ def _configure_logging(level, filename):
                         level=level)
     global LOG
     LOG = logging.getLogger(__name__)
-    handler = logging.StreamHandler()
-    formatter = logging.Formatter(DEFAULT_LOG_FORMAT)
-    handler.setFormatter(formatter)
-    LOG.addHandler(handler)
+    if os.environ.get('OS_LOG_CAPTURE') != '1':
+        handler = logging.StreamHandler()
+        formatter = logging.Formatter(DEFAULT_LOG_FORMAT)
+        handler.setFormatter(formatter)
+        LOG.addHandler(handler)
 
 
 def _load_config():
@@ -605,12 +632,13 @@ def _check_hostname():
 def _check_memory():
     """Check system memory
 
-    The undercloud will not run properly in less than 4 GB of memory.
+    The undercloud will not run properly in less than 8 GB of memory.
     This function verifies that at least that much is available before
     proceeding with install.
     """
     mem = psutil.virtual_memory()
-    total_mb = mem.total / 1024 / 1024
+    swap = psutil.swap_memory()
+    total_mb = (mem.total + swap.total) / 1024 / 1024
     if total_mb < REQUIRED_MB:
         LOG.error('At least %d MB of memory is required for undercloud '
                   'installation.  A minimum of 8 GB is recommended. '
@@ -678,22 +706,6 @@ def _validate_no_ip_change():
         raise validator.FailedValidation(message)
 
 
-def _validate_passwords_file():
-    """Disallow updates if the passwords file is missing
-
-    If the undercloud was already deployed, the passwords file needs to be
-    present so passwords that can't be changed are persisted.  If the file
-    is missing it will break the undercloud, so we should fail-fast and let
-    the user know about the problem.
-    """
-    if (os.path.isfile(os.path.expanduser('~/stackrc')) and
-            not os.path.isfile(PATHS.PASSWORD_PATH)):
-        message = ('The %s file is missing.  This will cause all service '
-                   'passwords to change and break the existing undercloud. ' %
-                   PATHS.PASSWORD_PATH)
-        raise validator.FailedValidation(message)
-
-
 def _validate_configuration():
     try:
         _check_hostname()
@@ -701,7 +713,6 @@ def _validate_configuration():
         _check_sysctl()
         _validate_network()
         _validate_no_ip_change()
-        _validate_passwords_file()
     except RuntimeError as e:
         LOG.error('An error occurred during configuration validation, '
                   'please check your host configuration and try again. '
@@ -771,7 +782,7 @@ def _generate_endpoints(instack_env):
             {'host': public_host, 'port': 8004, 'ssl_port': 13004},
             {'host': internal_host, 'port': 8004}),
         ('heat-ui-config',
-            '%s://%s:%d/heat/v1/%%(tenant_id)s',
+            '%s://%s:%d/heat/v1/%%(project_id)s',
             {'host': public_host, 'port': 3000, 'ssl_port': 443},
             {'host': internal_host, 'port': 3000}),
         ('neutron',
@@ -800,7 +811,7 @@ def _generate_endpoints(instack_env):
             {'host': internal_host, 'port': 5000},
             {'host': internal_host, 'port': 35357}),
         ('keystone-ui-config',
-            '%s://%s:%d/keystone/v2.0',
+            '%s://%s:%d/keystone/v3',
             {'host': public_host, 'port': 3000, 'ssl_port': 443},
             {'host': internal_host, 'port': 3000},
             {'host': internal_host, 'port': 35357}),
@@ -813,7 +824,7 @@ def _generate_endpoints(instack_env):
             {'host': public_host, 'port': 8080, 'ssl_port': 13808},
             {'host': internal_host, 'port': 8080}),
         ('swift-ui-config',
-            '%s://%s:%d/swift/v1/AUTH_%%(tenant_id)s',
+            '%s://%s:%d/swift/v1/AUTH_%%(project_id)s',
             {'host': public_host, 'port': 3000, 'ssl_port': 443},
             {'host': internal_host, 'port': 3000}),
         ('ironic',
@@ -926,41 +937,53 @@ def _write_password_file(instack_env):
                 LOG.info('Generated new password for %s', opt.name)
             instack_env[env_name] = value
             password_file.write('%s=%s\n' % (opt.name, value))
+    os.chmod(PATHS.PASSWORD_PATH, 0o600)
 
 
-def _member_role_exists(instack_env):
+def _member_role_exists():
     # This is a workaround for puppet removing the deprecated _member_
-    # role on upgrade - if it exists we must not remove role assignments
+    # role on upgrade - if it exists we must restore role assignments
     # or trusts stored in the undercloud heat will break
-    if not _stackrc_exists():
-        instack_env['MEMBER_ROLE_EXISTS'] = 'False'
-        return
-    user, password, tenant, auth_url = _get_auth_values()
-    role_exists = False
+    user, password, project, auth_url = _get_auth_values()
+    auth_kwargs = {
+        'auth_url': auth_url,
+        'username': user,
+        'password': password,
+        'project_name': project,
+        'project_domain_name': 'Default',
+        'user_domain_name': 'Default',
+    }
+    auth_plugin = ks_auth.Password(**auth_kwargs)
+    sess = session.Session(auth=auth_plugin)
+    disc = discover.Discover(session=sess)
+    c = disc.create_client()
     try:
-        # Note this is made somewhat verbose due to trying to handle
-        # any format auth_url (versionless, v2,0/v3 suffix)
-        auth_plugin_class = auth.get_plugin_class('password')
-        auth_kwargs = {
-            'auth_url': auth_url,
-            'username': user,
-            'password': password,
-            'project_name': tenant}
-        if 'v2.0' not in auth_url:
-            auth_kwargs.update({
-                'project_domain_name': 'default',
-                'user_domain_name': 'default'})
-        auth_plugin = auth_plugin_class(**auth_kwargs)
-        sess = session.Session(auth=auth_plugin)
-        disc = discover.Discover(session=sess)
-        c = disc.create_client()
-        role_names = [r.name for r in c.roles.list()]
-        role_exists = '_member_' in role_names
-    except ks_exceptions.ConnectionError:
-        # This will happen on initial deployment, assume False
-        # as no new deployments should have _member_
-        role_exists = False
-    instack_env['MEMBER_ROLE_EXISTS'] = six.text_type(role_exists)
+        member_role = [r for r in c.roles.list() if r.name == '_member_'][0]
+    except IndexError:
+        # Do nothing if there is no _member_ role
+        return
+    if c.version == 'v2.0':
+        client_projects = c.tenants
+    else:
+        client_projects = c.projects
+    admin_project = [t for t in client_projects.list() if t.name == 'admin'][0]
+    admin_user = [u for u in c.users.list() if u.name == 'admin'][0]
+    if c.version == 'v2.0':
+        try:
+            c.roles.add_user_role(admin_user, member_role, admin_project.id)
+            LOG.info('Added _member_ role to admin user')
+        except ks_exceptions.http.Conflict:
+            # They already had the role
+            pass
+    else:
+        try:
+            c.roles.grant(member_role,
+                          user=admin_user,
+                          project=admin_project.id)
+            LOG.info('Added _member_ role to admin user')
+        except ks_exceptions.http.Conflict:
+            # They already had the role
+            pass
 
 
 class InstackEnvironment(dict):
@@ -977,7 +1000,9 @@ class InstackEnvironment(dict):
     DYNAMIC_KEYS = {'INSPECTION_COLLECTORS', 'INSPECTION_KERNEL_ARGS',
                     'INSPECTION_NODE_NOT_FOUND_HOOK',
                     'TRIPLEO_INSTALL_USER', 'TRIPLEO_UNDERCLOUD_CONF_FILE',
-                    'TRIPLEO_UNDERCLOUD_PASSWORD_FILE', 'MEMBER_ROLE_EXISTS'}
+                    'TRIPLEO_UNDERCLOUD_PASSWORD_FILE',
+                    'ENABLED_POWER_INTERFACES',
+                    'ENABLED_MANAGEMENT_INTERFACES'}
     """The variables we calculate in _generate_environment call."""
 
     PUPPET_KEYS = DYNAMIC_KEYS | {opt.name.upper() for _, group in list_opts()
@@ -997,6 +1022,11 @@ class InstackEnvironment(dict):
         return super(InstackEnvironment, self).__setitem__(key, value)
 
 
+def _make_list(values):
+    """Generate a list suitable to pass to templates."""
+    return '[%s]' % ', '.join('"%s"' % item for item in values)
+
+
 def _generate_environment(instack_root):
     """Generate an environment dict for instack
 
@@ -1079,14 +1109,32 @@ def _generate_environment(instack_root):
 
     # Ensure correct rendering of the list and uniqueness of the items
     enabled_drivers = set(CONF.enabled_drivers)
+    enabled_hardware_types = set(CONF.enabled_hardware_types)
     if CONF.enable_node_discovery:
-        enabled_drivers.add(CONF.discovery_default_driver)
+        if (CONF.discovery_default_driver not in (enabled_drivers |
+                                                  enabled_hardware_types)):
+            enabled_drivers.add(CONF.discovery_default_driver)
         instack_env['INSPECTION_NODE_NOT_FOUND_HOOK'] = 'enroll'
     else:
         instack_env['INSPECTION_NODE_NOT_FOUND_HOOK'] = ''
 
-    instack_env['ENABLED_DRIVERS'] = (
-        '[%s]' % ', '.join('"%s"' % drv for drv in set(enabled_drivers)))
+    # In most cases power and management interfaces are called the same, so we
+    # use one variable for them.
+    enabled_interfaces = set()
+    if 'ipmi' in enabled_hardware_types:
+        enabled_interfaces.add('ipmitool')
+    if 'redfish' in enabled_hardware_types:
+        enabled_interfaces.add('redfish')
+
+    instack_env['ENABLED_DRIVERS'] = _make_list(enabled_drivers)
+    instack_env['ENABLED_HARDWARE_TYPES'] = _make_list(enabled_hardware_types)
+
+    enabled_interfaces = _make_list(enabled_interfaces)
+    instack_env['ENABLED_POWER_INTERFACES'] = enabled_interfaces
+    instack_env['ENABLED_MANAGEMENT_INTERFACES'] = enabled_interfaces
+
+    if CONF.docker_registry_mirror:
+        instack_env['DOCKER_REGISTRY_MIRROR'] = CONF.docker_registry_mirror
 
     instack_env['PUBLIC_INTERFACE_IP'] = instack_env['LOCAL_IP']
     instack_env['LOCAL_IP'] = instack_env['LOCAL_IP'].split('/')[0]
@@ -1116,8 +1164,6 @@ def _generate_environment(instack_root):
         instack_env['UNDERCLOUD_SERVICE_CERTIFICATE'] = (
             '/etc/pki/tls/certs/undercloud-%s.pem' % public_host)
 
-    _member_role_exists(instack_env)
-
     return instack_env
 
 
@@ -1232,14 +1278,14 @@ def _ensure_user_identity(id_path):
 def _get_auth_values():
     """Get auth values from stackrc
 
-    Returns the user, password, tenant and auth_url as read from stackrc,
+    Returns the user, password, project and auth_url as read from stackrc,
     in that order as a tuple.
     """
     user = _extract_from_stackrc('OS_USERNAME')
     password = _run_command(['sudo', 'hiera', 'admin_password']).rstrip()
-    tenant = _extract_from_stackrc('OS_TENANT_NAME')
+    project = _extract_from_stackrc('OS_PROJECT_NAME')
     auth_url = _extract_from_stackrc('OS_AUTH_URL')
-    return user, password, tenant, auth_url
+    return user, password, project, auth_url
 
 
 def _configure_ssh_keys(nova):
@@ -1283,14 +1329,6 @@ def _ensure_flavor(nova, name, profile=None):
     LOG.info(message, name, profile)
 
 
-def _stackrc_exists():
-    user_stackrc = os.path.expanduser('~/stackrc')
-    # We gotta check if the copying of stackrc has already been done.
-    if os.path.isfile('/root/stackrc') and os.path.isfile(user_stackrc):
-        return True
-    return False
-
-
 def _copy_stackrc():
     args = ['sudo', 'cp', '/root/stackrc', os.path.expanduser('~')]
     try:
@@ -1308,6 +1346,12 @@ def _clean_os_refresh_config():
     _run_command(args, name='Clean os-refresh-config')
 
 
+def _clean_os_collect_config():
+    occ_dir = '/var/lib/os-collect-config'
+    args = ['sudo', 'rm', '-fr', occ_dir]
+    _run_command(args, name='Clean os-collect-config')
+
+
 def _create_mistral_config_environment(instack_env, mistral):
     # Store the snmpd password in a Mistral environment so it can be accessed
     # by the Mistral actions.
@@ -1316,7 +1360,7 @@ def _create_mistral_config_environment(instack_env, mistral):
     env_name = "tripleo.undercloud-config"
     try:
         mistral.environments.get(env_name)
-    except mistralclient_base.APIException:
+    except ks_exceptions.NotFound:
         mistral.environments.create(
             name=env_name,
             variables=json.dumps({
@@ -1324,11 +1368,34 @@ def _create_mistral_config_environment(instack_env, mistral):
             }))
 
 
-def _create_default_plan(mistral, timeout=360):
+def _migrate_plans(mistral, swift, plans):
+    """Migrate plan environments from Mistral to Swift."""
+    plan_env_filename = 'plan-environment.yaml'
+
+    for plan in plans:
+        headers, objects = swift.get_container(plan)
+
+        if headers.get('x-container-meta-usage-tripleo') != 'plan':
+            continue
+
+        try:
+            swift.get_object(plan, plan_env_filename)
+        except swiftclient.ClientException:
+            LOG.info('Migrating environment for plan %s to Swift.' % plan)
+            env = mistral.environments.get(plan).variables
+            yaml_string = yaml.safe_dump(env, default_flow_style=False)
+            swift.put_object(plan, plan_env_filename, yaml_string)
+            # TODO(jpichon): delete env from Mistral once triple-common
+            # change merges (https://review.openstack.org/#/c/452291/).
+            # Right now plan operations still expect a Mistral environment.
+            # mistral.environments.delete(plan)
+
+
+def _create_default_plan(mistral, plans, timeout=360):
     plan_name = 'overcloud'
     queue_name = str(uuid.uuid4())
 
-    if plan_name in [env.name for env in mistral.environments.list()]:
+    if plan_name in plans:
         LOG.info('Not creating default plan "%s" because it already exists.',
                  plan_name)
         return
@@ -1366,10 +1433,12 @@ def _prepare_ssh_environment(mistral):
     mistral.executions.create('tripleo.validations.v1.copy_ssh_key')
 
 
-def _post_config_mistral(instack_env, mistral):
+def _post_config_mistral(instack_env, mistral, swift):
+    plans = [container["name"] for container in swift.get_account()[1]]
 
     _create_mistral_config_environment(instack_env, mistral)
-    _create_default_plan(mistral)
+    _migrate_plans(mistral, swift, plans)
+    _create_default_plan(mistral, plans)
 
     if CONF.enable_validations:
         _prepare_ssh_environment(mistral)
@@ -1377,13 +1446,23 @@ def _post_config_mistral(instack_env, mistral):
 
 def _post_config(instack_env):
     _copy_stackrc()
-    user, password, tenant, auth_url = _get_auth_values()
+    user, password, project, auth_url = _get_auth_values()
+    auth_kwargs = {
+        'auth_url': auth_url,
+        'username': user,
+        'password': password,
+        'project_name': project,
+        'project_domain_name': 'Default',
+        'user_domain_name': 'Default',
+    }
+    auth_plugin = ks_auth.Password(**auth_kwargs)
+    sess = session.Session(auth=auth_plugin)
     # TODO(andreykurilin): remove this check with support of novaclient 6.0.0
     if nc.__version__[0] == "6":
-        nova = novaclient.Client(2, user, password, tenant, auth_url=auth_url)
+        nova = novaclient.Client(2, user, password, project, auth_url=auth_url)
     else:
         nova = novaclient.Client(2, user, password, auth_url=auth_url,
-                                 project_name=tenant)
+                                 project_name=project)
 
     _configure_ssh_keys(nova)
     _delete_default_flavors(nova)
@@ -1398,11 +1477,13 @@ def _post_config(instack_env):
     mistral_url = instack_env['UNDERCLOUD_ENDPOINT_MISTRAL_PUBLIC']
     mistral = mistralclient.client(
         mistral_url=mistral_url,
-        username=user,
-        api_key=password,
-        project_name=tenant,
-        auth_url=auth_url)
-    _post_config_mistral(instack_env, mistral)
+        session=sess)
+    swift = swiftclient.Connection(
+        authurl=auth_url,
+        session=sess
+    )
+    _post_config_mistral(instack_env, mistral, swift)
+    _member_role_exists()
 
 
 def _handle_upgrade_fact(upgrade=False):
@@ -1458,6 +1539,7 @@ def install(instack_root, upgrade=False):
         LOG.info('Logging to %s', PATHS.LOG_FILE)
         _load_config()
         _clean_os_refresh_config()
+        _clean_os_collect_config()
         _validate_configuration()
         instack_env = _generate_environment(instack_root)
         _generate_init_data(instack_env)
@@ -1486,6 +1568,7 @@ def install(instack_root, upgrade=False):
         _post_config(instack_env)
         _run_command(['sudo', 'rm', '-f', '/tmp/svc-map-services'], None, 'rm')
     except Exception as e:
+        LOG.debug("An exception occurred", exc_info=True)
         LOG.error(FAILURE_MESSAGE,
                   {'undercloud_operation': undercloud_operation,
                    'exception': six.text_type(e),
